@@ -4,66 +4,67 @@ use strict;
 use warnings;
 use Carp;
 use DBI;
-require Exporter;
 
-our @ISA = qw(Exporter);
-
-our @EXPORT =
-	qw( insert update delete delete_all select create_table drop_table table err );
-
-our @EXPORT_OK =
-	qw( );
- 
-our %EXPORT_TAGS = (
-	#all => [ @EXPORT_OK ],
-);
-  
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
+my $td = {
+    week =>
+      { 0, 'Sun', 1, 'Mon', 2, 'Tue', 3, 'Wed', 4, 'Thu', 5, 'Fri', 6, 'Sat' },
+    month => {
+        '01' => 'Jan',
+        '02' => 'Feb',
+        '03' => 'Mar',
+        '04' => 'Apr',
+        '05' => 'May',
+        '06' => 'Jun',
+        '07' => 'Jul',
+        '08' => 'Aug',
+        '09' => 'Sep',
+        '10' => 'Oct',
+        '11' => 'Nov',
+        '12' => 'Dec',
+    }
+};
 
 sub new {
     my $class = shift;
     my $data  = shift;
     my $self;
-	my $attrs = {
-		AutoCommit => 0,
-		PrintError => 0,
-		RaiseError => 1,
-	};
-	
+    my $attrs = {
+        AutoCommit => 0,
+        PrintError => 0,
+        RaiseError => 1,
+    };
+
     if ( not ref $data eq 'HASH' ) {
         my $dbtype    = shift;
         my $tablename = shift;
         $self->{'dbname'}    = $data;
         $self->{'dbtype'}    = $dbtype ? $dbtype : "dbi:SQLite2:dbname";
         $self->{'tablename'} = $tablename;
-		$self->{'attrs'}     = $attrs;
+        $self->{'attrs'}     = $attrs;
     }
     else {
         $self = {
             dbname    => $data->{'DB'},
             dbtype    => $data->{'DSN'},
             tablename => $data->{'TABLE'},
-			attrs	  => ref $data->{'attrs'} eq 'HASH'
-							? $data->{'attrs'}
-							: $attrs
+            attrs     => ref $data->{'attrs'} eq 'HASH'
+            ? $data->{'attrs'}
+            : $attrs
         };
     }
-	
-	
-	$class eq __PACKAGE__
+
+    $class eq __PACKAGE__
       or croak _err_msg("constructor not called as class method");
 
-    -e $self->{'dbname'}	
-	  or croak _err_msg("no database defined");
+    -e $self->{'dbname'}
+      or croak _err_msg("no database defined");
     -f $self->{'dbname'}
       or croak _err_msg("no such database $self->{dbname}");
 
-    $self->{'dbh'} = DBI->connect(
-        qq/$self->{dbtype}=$self->{dbname}/,
-		q//, q//,
-		$self->{'attrs'}
-    );
+    $self->{'dbh'} = DBI->connect( qq/$self->{dbtype}=$self->{dbname}/,
+        q//, q//, $self->{'attrs'} );
 
     $self->{'BEGIN'}    = sub { $self->{'dbh'}->begin_work };
     $self->{'COMMIT'}   = sub { $self->{'dbh'}->commit };
@@ -73,45 +74,106 @@ sub new {
 
 }
 
-sub table:lvalue {
+sub table : lvalue {
     $_[1]
-      ? $_[0]->{q{tablename}} = $_[1]
+      ? $_[0]->{q{tablename}} =
+        $_[1]
       : $_[0]->{q{tablename}};
 }
 
+sub tables {
+    my $self = shift;
+    no strict 'refs';
+
+    map { $_->[0] } $self->select(
+        q/
+            SELECT name FROM 
+                (SELECT * FROM sqlite_master UNION ALL
+                SELECT * FROM sqlite_temp_master)
+            WHERE type='table'
+            ORDER BY name;
+    /
+    );
+}
+
+sub localtime {
+    my $self = shift;
+    my $ps   = shift;
+
+    if ($ps) {
+        my @td = split q/ /, $self->select(
+            q/
+        	select strftime("%w %m %d %H:%M:%S %Y",'now','localtime');
+    	/
+        )->[0];
+        $td[0] = $td->{q/week/}{ $td[0] };
+        $td[1] = $td->{q/month/}{ $td[1] };
+
+        return join q/ /, @td;
+    }
+
+    return join q/ /, map { $_->[0] } $self->select(
+        q/
+		select datetime('now','localtime');
+	/
+    );
+}
+
+sub time {
+    my $self = shift;
+
+    return join q/ /, map { $_->[0] } $self->select(
+        q/
+        	select strftime("%s",'now');
+    	/
+    );
+
+}
+
+sub time_ahead {
+    my $self  = shift;
+    my $ahead = "@_";
+
+    return join q/ /, map { $_->[0] } $self->select(
+        qq/
+       		select datetime('now','localtime',$ahead);
+	/
+    );
+
+}
+
 sub create_table {
     my $self      = shift;
     my $tablename = shift || $self->{tablename};
 
     $self->do(
-		qq/
-			CREATE TABLE $tablename ( @_ );
-		/
+        qq/
+		CREATE TABLE $tablename ( @_ );
+	/
     );
 }
 
 sub alter {
     my $self  = shift;
     my $query = shift;
-	
-    $self->_check_table;
 
+    $self->_check_table;
     $self->do(
         qq/
-			ALTER TABLE $self->{tablename} $query;
-		/
+		ALTER TABLE $self->{tablename} $query;
+	/
     );
 }
 
 sub drop_table {
-    my $self = shift;
-	my $tablename = shift || $self->{tablename};
+    my $self      = shift;
+    my $tablename = shift || $self->{tablename};
 
     $self->do(
         qq/
-			DROP TABLE $tablename; 
-			VACUUM;
-		/
+		DROP TABLE $tablename; 
+		VACUUM;
+	/
     );
 }
 
@@ -131,17 +193,17 @@ sub insert {
         my $prep_data = join( ',', split '', ( '?' x @$columns ) );
         $sth = $self->{q{dbh}}->prepare(
             qq/
-				INSERT INTO $self->{tablename} ($prep_columns) 
-				VALUES ($prep_data);
-			/	
+			INSERT INTO $self->{tablename} ($prep_columns) 
+			VALUES ($prep_data);
+		/
         );
     }
     else {
-        my $prep_data = join(',', (split '', ('?' x @{ $data->[0] })));
+        my $prep_data = join( ',', ( split '', ( '?' x @{ $data->[0] } ) ) );
         $sth = $self->{q{dbh}}->prepare(
             qq/
-				INSERT INTO $self->{tablename} VALUES ($prep_data);
-			/
+			INSERT INTO $self->{tablename} VALUES ($prep_data);
+		/
         );
     }
 
@@ -157,26 +219,49 @@ sub insert {
 sub replace {
     my $self  = shift;
     my $query = shift;
-    my $result;
 
     $self->_check_table;
     $self->do(
         qq/
-			REPLACE INTO $self->{tablename} $query;
-		/
+		REPLACE INTO $self->{tablename} $query;
+	/
+    );
+}
+
+sub create_view {
+    my $self      = shift;
+    my $view_name = shift;
+    my $select    = shift;
+
+    $self->_check_table;
+    $self->do(
+        qq/
+		CREATE VIEW $view_name AS $select;
+	/
+    );
+}
+
+sub drop_view {
+    my $self      = shift;
+    my $view_name = shift;
+
+    $self->_check_table;
+    $self->do(
+        qq/
+		DROP VIEW $view_name;
+	/
     );
 }
 
 sub delete {
     my $self  = shift;
     my $query = shift;
-    my $result;
 
     $self->_check_table;
     $self->do(
         qq/
-			DELETE FROM $self->{tablename} $query;
-		/
+		DELETE FROM $self->{tablename} $query;
+	/
     );
 }
 
@@ -189,11 +274,10 @@ sub update {
     my $query = shift;
 
     $self->_check_table;
-
     $self->do(
         qq/
-			UPDATE $self->{tablename} SET $query;
-		/
+		UPDATE $self->{tablename} SET $query;
+	/
     );
 }
 
@@ -204,11 +288,11 @@ sub select {
     my $result;
 
     $self->_check_table;
-	
-	$query ||= qq/SELECT * FROM $self->{q{tablename}}/;
-	$query =~ s/^\s*(\w*\s*ALL\s*)/ * /i;
-    
-	if ( not $query =~ /^\s*SELECT\s+/i ) {
+
+    $query ||= qq/SELECT * FROM $self->{q{tablename}}/;
+    $query =~ s/^\s*(\w*\s*ALL\s*)/ * /i;
+
+    if ( not $query =~ /^\s*SELECT\s+/i ) {
         if ( $query =~ /^\s*\*/ ) {
             $query =~ s/^\s*\*(.*)/SELECT * FROM 
 				$self->{q{tablename}} $1/;
@@ -219,6 +303,8 @@ sub select {
         }
     }
 
+    local $self->{q{dbh}}->{q{RaiseError}} = 1;
+
     if (wantarray) {
         if ( my @data = @{ $self->{q{dbh}}->selectall_arrayref($query) } ) {
             $self->_END_;
@@ -228,7 +314,7 @@ sub select {
         }
     }
     else {
-		local $_ = join('_', $query);
+        local $_ = join( '_', $query );
         if ( not $self->{$_} ) {
             $self->{$_} = $self->{q{dbh}}->prepare($query);
             $self->{$_}->execute;
@@ -237,19 +323,19 @@ sub select {
         else {
             $result = [ $self->{$_}->fetchrow_array ];
         }
-	
-		$self->{q/select/}->{q/last/} = $result;
-		
+
+        $self->{q/select/}->{q/last/} = $result;
+
         return @$result
           ? ref $type eq 'SCALAR'
-          	? $result->[0]
-          	: $result
+             ? $result->[0]
+             : $result
           : undef $self->{$_};
     }
 }
 
 sub last {
-	shift->{q/select/}->{q/last/};
+    shift->{q/select/}->{q/last/};
 }
 
 sub count {
@@ -261,12 +347,12 @@ sub count {
     my $count = $query
       ? $self->{q{dbh}}->selectall_arrayref(
         qq/
-					SELECT count(*) FROM $self->{tablename} $query;
+			SELECT count(*) FROM $self->{tablename} $query
 		/
       )
       : $self->{q{dbh}}->selectall_arrayref(
         qq/
-					SELECT count(*) FROM $self->{tablename};	
+			SELECT count(*) FROM $self->{tablename}
 		/
       );
 
@@ -279,10 +365,6 @@ sub sum {
     shift->count(@_);
 }
 
-sub _BEGIN_ {
-    shift->{q{BEGIN}}->();
-}
-
 sub _END_ {
     my $self = shift;
 
@@ -298,32 +380,29 @@ sub do {
 sub _do_ {
     my $self  = shift;
     my $query = "@_";
-	my $affected;
-	
-	local $self->{q{dbh}}->{q{RaiseError}} = 1;
+    my $affected;
+
     eval {
         $affected = $self->{q{dbh}}->do($query);
         $self->{q{COMMIT}}->();
     };
 
     if ( $self->{q{dbh}}->errstr ) {
-		$self->err = $@;
-        eval { 
-			$self->{q{ROLLBACK}}->() 
-		};
-		if ( $self->{q{dbh}}->{q{RaiseError}} ) {
-			croak $self->err;
-		}
-		else {
-			carp $self->err;
-			return undef;
-		}
+        $self->err = $@;
+        eval { $self->{q{ROLLBACK}}->() };
+        if ( $self->{q{dbh}}->{q{RaiseError}} ) {
+            croak $self->err;
+        }
+        else {
+            carp $self->err;
+            return undef;
+        }
+    }
+    else {
+        return $affected == 0
+          ? "0E0"
+          : $affected;
     }
-	else {
-		return $affected == 0
-			? "0E0"
-			: $affected;
-	}
 }
 
 sub _check_table {
@@ -336,20 +415,20 @@ sub _err_msg {
     __PACKAGE__ . q/:/ . (caller)[2] . q/:/ . __LINE__ . q/: / . "@_";
 }
 
-sub err:lvalue {
-	$_[1]
-		? $_[0]->{q{err}} = $_[1]
-		: $_[0]->{q{err}};
+sub err : lvalue {
+    $_[1]
+      ? $_[0]->{q{err}} = $_[1]
+      : $_[0]->{q{err}};
 }
 
 sub AUTOLOAD {
     my $self  = shift;
     my $query = shift;
     my $tmp   = shift;
-    
-	$tmp and $query = $tmp;
-	$query or $query = q{};
-	
+
+    $tmp and $query = $tmp;
+    $query or $query = q{};
+
     our $AUTOLOAD;
     ( my $method = $AUTOLOAD ) =~ s/.*:://s;
 
@@ -367,9 +446,9 @@ sub AUTOLOAD {
 }
 
 sub DESTROY {
-	my $self = shift;
-	$self->{q{dbh}} = undef;
-	$self = undef;
+    my $self = shift;
+    $self->{q{dbh}} = undef;
+    $self = undef;
 }
 
 1
@@ -498,6 +577,13 @@ This is the table which all methods use by default.
  $sql->table($tablename); # sets and returns the default table name
  $sq->table = $tablename;
 
+=item C<tables>
+
+Lists the tables in the database. Returns a list with the tables or true on sucess. Returns undef on failure or raises fatal error exception according to C<$dbh> C<RaiseError> attribute.
+
+ @tables = $sql->tables();
+
+
 =item C<create_table>
 
 Creates table.
@@ -536,6 +622,29 @@ table name.
 Returns true on success. Returns undef on failure or raises fatal error exception
 according to C<$dbh> C<RaiseError> attribute.
 
+=item C<create_view>
+
+Creates a view which is a named L<select> statement. Requires two arguments, the view's name and
+a SELECT statement. View is created in the object's database. It is read-only. It is removed with L<drop_view>.
+
+ $sql->create_view("view_name", q/
+     SELECT id FROM $tablename
+      WHERE ...
+ /);
+ $sql->table = "view_name";
+ $sql->select();
+
+Returns true on success otherwise returns undef or raises fatal error exception according to C<$dbh> C<RaiseError>
+attribute.
+
+=item C<drop_view>
+
+Removes a view. Requires the view's name as argument. 
+
+ $sql->drop_view("view_name");
+
+Returns true on success otherwise returns undef or raises fatal error exception according to C<$dbh> C<RaiseError>
+attribute.
 
 =back
 
@@ -547,7 +656,7 @@ according to C<$dbh> C<RaiseError> attribute.
 
 Inserts data. Takes array references, the columns and the data to be inserted into 
 these columns. The data array (which must be array of array references) can be given alone
-in which case each element is expected to have refer to the same number as the columns in
+in which case each element is expected to refer to the same number as the columns in
 the default sql table. Returns the number of affected rows. Returns false unless inserted rows.
 Returns undef on failure or raises fatal error exception according to C<$dbh> C<RaiseError> 
 attribute.
@@ -618,8 +727,8 @@ select methods with the proper sql syntax.
  
 Returns array containing array references to each row in list context. In scalar context
 returns result as string if one column selected, otherwise reference to the row fetched.
-Returns undef at the end while in scalat context. Returns undef on failure or raises fatal 
-error exception according to C<$dbh> C<RaiseError> attribute.
+Returns undef at the end while in scalar context. Raises fatal error exception on failure
+(C<local>ized RaiseError attribute is set to true).
 
 =item C<count>
 
@@ -627,12 +736,31 @@ Implements rows counting. Returns undef on failure or raises fatal error excepti
 according to C<$dbh> C<RaiseError> attribute.
 
  $sql->count;
- $sql->count(q/SELECT count(*) FROM tablename/);
+ $sql->do(q/SELECT count(*) FROM tablename/);
 
 =item C<sum>
 
 Implements C<count> method.
 
+=item C<time>
+
+SQLite2 datetime('now') function, identical to Perl time() function.
+
+=item C<localtime>
+
+SQLite2 datetime('now','localtime') function. Format of the return is the same as Perl localtime() in scalar context
+if positive argument is given, otherwise returns in the default SQLite2 format: yyyy-mm-dd HH:MM:SS.
+
+ $sql->localtime(); # 2006-03-27 14:33:32
+ $sql->localtime(1); # Mon Mar 27 14:35:25 2006
+
+=item C<time_ahead>
+
+SQLite2 datetime('now','localtime','+.. time', ...) function. Takes y/m/d/H/M/S arguments.
+Returns the current time plus the time given in the default format: yyyy-mm-dd HH:MM:SS.
+
+ $s1->time_ahead(q/'+5 minutes','+2 seconds','+3 hours','+9 days','+9 months','+15 years'/);
+
 =item C<do>
  
 Calls C<$dbh> C<do> method. Useful for I<non>-C<SELECT> arbitrary statements which will not be 
